/*
 * csta-m.c : Addons for the CSTA standard files (.[ch] from .asn1)
 *
 * Copyright (C) 2000 Alcve and Julien Gaulmin <julien.gaulmin@fr.alcove.com>
 *
 * Based on work from :
 * Francisco Espinoza Junior and Philippe de M. Sevestre
 * Dedalus Engenharia S/C Ltda - Convenio GMK/FDTE
 *
 * Contributors :
 *   Jecode <comptel@crosswinds.net>
 *   - fixed the Service Response decoding error problem
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */

#include "csta2.h"
#include "csta-m.h"
#include "log.h"


/* Registers table of pending invocations */
static struct InvokeEntry {
	int used;		/* 0 == free; !0 == occupied */
	AsnInt invokeID;	/* Primary key */
	CSTAAnyID anyID;
} invokeTable[MAX_INVOKE];


/* Auxiliary routine to "swallow" the rest of the packet */
void EatAny(BUF_TYPE b, void *value, AsnLen *bytesDecoded, ENV_TYPE env)
{
	int i;
	
	while (!BufReadError(b)) {
		BufGetByte(b);
		*bytesDecoded = *bytesDecoded + 1;
	}
}


/* Initialize the hashing table for ANYs managment */
void CSTAM_InitAny()
{
	int i;
	static struct {
		CSTAAnyID intVal;
		unsigned int size;
		EncodeFcn encode;
		DecodeFcn decode;
		PrintFcn print;
		FreeFcn free;
	} any_data[] = {
	/* ANY in the ROIVapdu */
	ALTERNATE_CALL,
		sizeof(AlternateCallArgument),
		(EncodeFcn) BEncAlternateCallArgument,
		(DecodeFcn) BDecAlternateCallArgument,
		(PrintFcn) PrintAlternateCallArgument,
		(FreeFcn) FreeAlternateCallArgument,
		
	ANSWER_CALL,
		sizeof(AnswerCallArgument),
		(EncodeFcn) BEncAnswerCallArgument,
		(DecodeFcn) BDecAnswerCallArgument,
		(PrintFcn) PrintAnswerCallArgument,
		(FreeFcn) FreeAnswerCallArgument,
		
        CLEAR_CONNECTION,
        	sizeof(ClearConnectionArgument),
        	(EncodeFcn) BEncClearConnectionArgument,
        	(DecodeFcn) BDecClearConnectionArgument,
        	(PrintFcn) PrintClearConnectionArgument,
        	(FreeFcn) FreeClearConnectionArgument,
        	
        CONFERENCE_CALL,
        	sizeof(ConferenceCallArgument),
        	(EncodeFcn) BEncConferenceCallArgument,
        	(DecodeFcn) BDecConferenceCallArgument,
        	(PrintFcn) PrintConferenceCallArgument,
        	(FreeFcn) FreeConferenceCallArgument,	
        
        CONSULTATION_CALL,
        	sizeof(ConsultationCallArgument),
        	(EncodeFcn) BEncConsultationCallArgument,
        	(DecodeFcn) BDecConsultationCallArgument,
        	(PrintFcn) PrintConsultationCallArgument,
        	(FreeFcn) FreeConsultationCallArgument,
        	
        DIVERT_CALL,
        	sizeof(DivertCallArgument),
        	(EncodeFcn) BEncDivertCallArgument,
        	(DecodeFcn) BDecDivertCallArgument,
        	(PrintFcn) PrintDivertCallArgument,
        	(FreeFcn) FreeDivertCallArgument,
        
        MAKE_CALL,
        	sizeof(MakeCallArgument),
        	(EncodeFcn) BEncMakeCallArgument,
        	(DecodeFcn) BDecMakeCallArgument,
        	(PrintFcn) PrintMakeCallArgument,
        	(FreeFcn) FreeMakeCallArgument,
        
        QUERY_DEVICE,
        	sizeof(QueryDeviceArgument),
        	(EncodeFcn) BEncQueryDeviceArgument,
        	(DecodeFcn) BDecQueryDeviceArgument,
        	(PrintFcn) PrintQueryDeviceArgument,
        	(FreeFcn) FreeQueryDeviceArgument,
        	
        RETRIEVE_CALL,
        	sizeof(RetrieveCallArgument),
        	(EncodeFcn) BEncRetrieveCallArgument,
        	(DecodeFcn) BDecRetrieveCallArgument,
        	(PrintFcn) PrintRetrieveCallArgument,
        	(FreeFcn) FreeRetrieveCallArgument,
        	
        SET_FEATURE,
        	sizeof(SetFeatureArgument),
        	(EncodeFcn) BEncSetFeatureArgument,
        	(DecodeFcn) BDecSetFeatureArgument,
        	(PrintFcn) PrintSetFeatureArgument,
        	(FreeFcn) FreeSetFeatureArgument,
        	
        TRANSFER_CALL,
        	sizeof(TransferCallArgument),
        	(EncodeFcn) BEncTransferCallArgument,
        	(DecodeFcn) BDecTransferCallArgument,
        	(PrintFcn) PrintTransferCallArgument,
        	(FreeFcn) FreeTransferCallArgument,
        	
        EVENT_REPORT,
        	sizeof(CSTAEventReportArgument),
        	(EncodeFcn) BEncCSTAEventReportArgument,
        	(DecodeFcn) BDecCSTAEventReportArgument,
        	(PrintFcn) PrintCSTAEventReportArgument,
        	(FreeFcn) FreeCSTAEventReportArgument,
        
        MONITOR_START,
        	sizeof(MonitorStartArgument),
        	(EncodeFcn) BEncMonitorStartArgument,
        	(DecodeFcn) BDecMonitorStartArgument,
        	(PrintFcn) PrintMonitorStartArgument,
        	(FreeFcn) FreeMonitorStartArgument,
        	
        MONITOR_STOP,
        	sizeof(MonitorStopArgument),
        	(EncodeFcn) BEncMonitorStopArgument,
        	(DecodeFcn) BDecMonitorStopArgument,
        	(PrintFcn) PrintMonitorStopArgument,
        	(FreeFcn) FreeMonitorStopArgument,
        
        SYSTEM_STATUS,
        	sizeof(SystemStatusArgument),
        	(EncodeFcn) BEncSystemStatusArgument,
        	(DecodeFcn) BDecSystemStatusArgument,
        	(PrintFcn) PrintSystemStatusArgument,
        	(FreeFcn) NULL,

	(ALTERNATE_CALL << 16),
		sizeof(AlternateCallResult),
		(EncodeFcn) BEncAlternateCallResult,
		(DecodeFcn) BDecAlternateCallResult,
		(PrintFcn) PrintAlternateCallResult,
		(FreeFcn) FreeAlternateCallResult,
		
	(ANSWER_CALL << 16),
		sizeof(AnswerCallResult),
		(EncodeFcn) BEncAnswerCallResult,
		(DecodeFcn) BDecAnswerCallResult,
		(PrintFcn) PrintAnswerCallResult,
		(FreeFcn) FreeAnswerCallResult,
		
        (CLEAR_CONNECTION << 16),
        	sizeof(ClearConnectionResult),
        	(EncodeFcn) BEncClearConnectionResult,
        	(DecodeFcn) BDecClearConnectionResult,
        	(PrintFcn) PrintClearConnectionResult,
        	(FreeFcn) FreeClearConnectionResult,
        	
        (CONFERENCE_CALL << 16),
        	sizeof(ConferenceCallResult),
        	(EncodeFcn) BEncConferenceCallResult,
        	(DecodeFcn) BDecConferenceCallResult,
        	(PrintFcn) PrintConferenceCallResult,
        	(FreeFcn) FreeConferenceCallResult,	
        
        (CONSULTATION_CALL << 16),
        	sizeof(ConsultationCallResult),
        	(EncodeFcn) BEncConsultationCallResult,
        	(DecodeFcn) BDecConsultationCallResult,
        	(PrintFcn) PrintConsultationCallResult,
        	(FreeFcn) FreeConsultationCallResult,
        	
        (DIVERT_CALL << 16),
        	sizeof(DivertCallResult),
        	(EncodeFcn) BEncDivertCallResult,
        	(DecodeFcn) BDecDivertCallResult,
        	(PrintFcn) PrintDivertCallResult,
        	(FreeFcn) FreeDivertCallResult,
        
        (MAKE_CALL << 16),
        	sizeof(MakeCallResult),
        	(EncodeFcn) BEncMakeCallResult,
        	(DecodeFcn) BDecMakeCallResult,
        	(PrintFcn) PrintMakeCallResult,
        	(FreeFcn) FreeMakeCallResult,
        
        (QUERY_DEVICE << 16),
        	sizeof(QueryDeviceResult),
        	(EncodeFcn) BEncQueryDeviceResult,
        	(DecodeFcn) BDecQueryDeviceResult,
        	(PrintFcn) PrintQueryDeviceResult,
        	(FreeFcn) FreeQueryDeviceResult,
        	
        (RETRIEVE_CALL << 16),
        	sizeof(RetrieveCallResult),
        	(EncodeFcn) BEncRetrieveCallResult,
        	(DecodeFcn) BDecRetrieveCallResult,
        	(PrintFcn) PrintRetrieveCallResult,
        	(FreeFcn) FreeRetrieveCallResult,
        	
        (SET_FEATURE << 16),
        	sizeof(SetFeatureResult),
        	(EncodeFcn) BEncSetFeatureResult,
        	(DecodeFcn) BDecSetFeatureResult,
        	(PrintFcn) PrintSetFeatureResult,
        	(FreeFcn) FreeSetFeatureResult,
        	
        (TRANSFER_CALL << 16),
        	sizeof(TransferCallResult),
        	(EncodeFcn) BEncTransferCallResult,
        	(DecodeFcn) BDecTransferCallResult,
        	(PrintFcn) PrintTransferCallResult,
        	(FreeFcn) FreeTransferCallResult,
        	
        (MONITOR_START << 16),
        	sizeof(MonitorStartResult),
        	(EncodeFcn) BEncMonitorStartResult,
        	(DecodeFcn) BDecMonitorStartResult,
        	(PrintFcn) PrintMonitorStartResult,
        	(FreeFcn) FreeMonitorStartResult,
        	
        (MONITOR_STOP << 16),
        	sizeof(MonitorStopResult),
        	(EncodeFcn) BEncMonitorStopResult,
        	(DecodeFcn) BDecMonitorStopResult,
        	(PrintFcn) PrintMonitorStopResult,
        	(FreeFcn) FreeMonitorStopResult,
        
        (SYSTEM_STATUS << 16),
        	sizeof(SystemStatusResult),
        	(EncodeFcn) BEncSystemStatusResult,
        	(DecodeFcn) BDecSystemStatusResult,
        	(PrintFcn) PrintSystemStatusResult,
        	(FreeFcn) NULL,


	/* ANY in the EventReport : virtual !!! */
        R_CONFERENCED,
        	sizeof(ConferencedEvent),
        	(EncodeFcn) BEncConferencedEvent,
        	(DecodeFcn) BDecConferencedEvent,
        	(PrintFcn) PrintConferencedEvent,
        	(FreeFcn) FreeConferencedEvent,
        	
        R_CONNECTIONCLEARED,
        	sizeof(ConnectionClearedEvent),
        	(EncodeFcn) BEncConnectionClearedEvent,
        	(DecodeFcn) BDecConnectionClearedEvent,
        	(PrintFcn) PrintConnectionClearedEvent,
        	(FreeFcn) FreeConnectionClearedEvent,
        	
        R_DELIVERED,
        	sizeof(DeliveredEvent),
        	(EncodeFcn) BEncDeliveredEvent,
        	(DecodeFcn) BDecDeliveredEvent,
        	(PrintFcn) PrintDeliveredEvent,
        	(FreeFcn) FreeDeliveredEvent,

        R_DIVERTED,
        	sizeof(DivertedEvent),
        	(EncodeFcn) BEncDivertedEvent,
        	(DecodeFcn) BDecDivertedEvent,
        	(PrintFcn) PrintDivertedEvent,
        	(FreeFcn) FreeDivertedEvent,
        	
        R_ESTABLISHED,
        	sizeof(EstablishedEvent),
        	(EncodeFcn) BEncEstablishedEvent,
        	(DecodeFcn) BDecEstablishedEvent,
        	(PrintFcn) PrintEstablishedEvent,
        	(FreeFcn) FreeEstablishedEvent,
        	
        R_FAILED,
        	sizeof(FailedEvent),
        	(EncodeFcn) BEncFailedEvent,
        	(DecodeFcn) BDecFailedEvent,
        	(PrintFcn) PrintFailedEvent,
        	(FreeFcn) FreeFailedEvent,
        	
        R_HELD,
        	sizeof(HeldEvent),
        	(EncodeFcn) BEncHeldEvent,
        	(DecodeFcn) BDecHeldEvent,
        	(PrintFcn) PrintHeldEvent,
        	(FreeFcn) FreeHeldEvent,
        	
        R_NETWORKREACHED,
        	sizeof(NetworkReachedEvent),
        	(EncodeFcn) BEncNetworkReachedEvent,
        	(DecodeFcn) BDecNetworkReachedEvent,
        	(PrintFcn) PrintNetworkReachedEvent,
        	(FreeFcn) FreeNetworkReachedEvent,

        R_QUEUED,
		sizeof(QueuedEvent),
        	(EncodeFcn) BEncQueuedEvent,
        	(DecodeFcn) BDecQueuedEvent,
        	(PrintFcn) PrintQueuedEvent,
        	(FreeFcn) FreeQueuedEvent,
	
        R_RETRIEVED,
        	sizeof(RetrievedEvent),
        	(EncodeFcn) BEncRetrievedEvent,
        	(DecodeFcn) BDecRetrievedEvent,
        	(PrintFcn) PrintRetrievedEvent,
        	(FreeFcn) FreeRetrievedEvent,
        	
        R_SERVICEINITIATED,
        	sizeof(ServiceInitiatedEvent),
        	(EncodeFcn) BEncServiceInitiatedEvent,
        	(DecodeFcn) BDecServiceInitiatedEvent,
        	(PrintFcn) PrintServiceInitiatedEvent,
        	(FreeFcn) FreeServiceInitiatedEvent,
        	
        R_TRANSFERRED,
        	sizeof(TransferedEvent),
        	(EncodeFcn) BEncTransferedEvent,
        	(DecodeFcn) BDecTransferedEvent,
        	(PrintFcn) PrintTransferedEvent,
        	(FreeFcn) FreeTransferedEvent,

	R_BACKINSERVICE,
        	sizeof(BackInServiceEvent),
        	(EncodeFcn) BEncBackInServiceEvent,
        	(DecodeFcn) BDecBackInServiceEvent,
        	(PrintFcn) PrintBackInServiceEvent,
        	(FreeFcn) FreeBackInServiceEvent,

	R_OUTOFSERVICE,
        	sizeof(OutOfServiceEvent),
        	(EncodeFcn) BEncOutOfServiceEvent,
        	(DecodeFcn) BDecOutOfServiceEvent,
        	(PrintFcn) PrintOutOfServiceEvent,
        	(FreeFcn) FreeOutOfServiceEvent,

	R_ORIGINATED,
        	sizeof(OriginatedEvent),
        	(EncodeFcn) BEncOriginatedEvent,
        	(DecodeFcn) BDecOriginatedEvent,
        	(PrintFcn) PrintOriginatedEvent,
        	(FreeFcn) FreeOriginatedEvent,

	R_PRIVATE,
		16,	/* Size fake */
		NULL,
		(DecodeFcn) EatAny,
		NULL,
		NULL
	};

	
	/* Install everything in the table */
	for (i = 0; i < sizeof(any_data)/sizeof(any_data[0]); i++) {
		InstallAnyByInt(
			any_data[i].intVal,
			any_data[i].intVal,
			any_data[i].size,
			(EncodeFcn) any_data[i].encode,
			(DecodeFcn) any_data[i].decode,
			(FreeFcn)   any_data[i].free,
			(PrintFcn)  any_data[i].print
			);	
	}
}


/* Consults the table of hanging invocations and obtains the type of data
 * associated. If InvokeID is found, it automatically remove the registration
 * from the table */
CSTAAnyID CSTAM_InvokeIDToAnyID(AsnInt invokeID)
{
	int i;
	
	for (i = 0; i < MAX_INVOKE; i++) {
		if (invokeTable[i].used == 1 && invokeTable[i].invokeID == invokeID) {
			/* Free invocation */
			invokeTable[i].used = 0;
			return invokeTable[i].anyID;
		}
	}
	
	fprintf(stderr, "Warning: InvokeID %d not found\n", invokeID);
	return (CSTAAnyID)0;
}


/* Auxiliary routines for manipulating the table of invocations. */
void CSTAM_InvokeTableInit()
{
	int i;
	
	for (i = 0; i < MAX_INVOKE; i++) {
		invokeTable[i].used = 0;
	}
}


void CSTAM_InvokeTableTerm()
{
	return ;
}


int CSTAM_InvokeRegister(AsnInt invokeID, CSTAAnyID anyID) 
{
	int i;
	
	for (i = 0; i < MAX_INVOKE; i++) {
		if (invokeTable[i].used == 0) {
			invokeTable[i].used = 1;
			invokeTable[i].invokeID = invokeID;
			invokeTable[i].anyID = anyID;
			return 1;
		}
	}

	logmsg(LOG_ERR, "InvokeRegister: Error placing the slot");
	return 0;
}


/* Only keeped for compatibility with old code. Use InvokeIDToAnyID instead */
void CSTAM_InvokeUnRegister(AsnInt invokeID)
{
	int i;

#if 0
	for (i = 0; i < MAX_INVOKE; i++) {
		if (invokeTable[i].invokeID == invokeID  {
			invokeTable[i].used = 0;
			return;
		}
	}
#endif
}

