/*
 * cstatest.c : test program of the BER routines for CSTA
 *
 * Copyright (C) 2000 Alcve and Julien Gaulmin <julien.gaulmin@fr.alcove.com>
 *
 * Based on work from :
 * Francisco Espinoza Junior and Philippe de M. Sevestre
 * Dedalus Engenharia S/C Ltda - Convenio GMK/FDTE
 *
 * Contributors :
 *   Jecode <comptel@crosswinds.net>
 *   - added the ConnectionID argument in testAnswerCall()
 *   - added the SystemStatus Event handling in dump()
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */

#include <setjmp.h>
#include <poll.h>
#include <signal.h>
#include <time.h>
#include <string.h>

#include "csta2.h"
#include "log.h"
#include "tspsock.h"


/* Options */
#define DEBUG_CSTATEST
//#define IS_SERVER


/* API */
void testMonitorStart(char *deviceNum);
void testMonitorStop(char *crossRefID);
void testMakeCall(char *from, char *to);
void testDivertCall(char *from, char *to);
void testTranferCall(char *heldDevice, char *connDevice);
void testQueryDevice(char *deviceNum);
void testAnswerCall(char *deviceNum, char *refid);
void ACSEassociation();


/* Prototypes */
void processAPDU(void *args, AsnInt serviceID);
void poll_input();
void setoctet(void *octet, char *s);
void getstring(char *prompt, char *buf, int size);
int getint(char *prompt);
int gethex(char *prompt);
#ifdef IS_SERVER
int do_server();
#endif
int dump();
void end();


/* Global variable */
static char scratch[8192];
static int gRequestID = 0x1000;
static int fdcsta;
#ifdef IS_SERVER
static int fdserver;
#endif
static time_t tm;
static char *logpath = "/import/cti/cti-data/csta.out";
static FILE *logfile;


/* Constantes and macros */
#define REQUEST_ID	gRequestID
#define NEXT_REQUESTID	++gRequestID
#define INFTIM		-1 
#define MAX_NUM_SIZE	20 
 
 
int main(int argc, char **argv)
{
	int test;

	/* Setup exit signals */
	signal(SIGTERM, end);
	signal(SIGQUIT, end);
	signal(SIGINT, end);

	/* Set log level and output stream */
	logsetlevel(LOG_DEBUG);
	loginit(stderr);

	/* Check args */
	if (argc != 3) {
		printf("Usage: cstatest <host> <port>\n");
		exit(-1);
	}

	/* Open TCP socket for the CSTA server */
	fdcsta = tsp_open_socket("tcp", argv[1], argv[2], CLIENT);
	if (fdcsta < 0) {
		perror("cstatest.c: main(): tsp_open_socket(CLIENT)");
		exit(-1);
	}

#ifdef IS_SERVER
	/* Open TCP socket for clients */
	fdserver = tsp_open_socket("udp", NULL, "3333", SERVER);
	if (fdserver < 0) {
		perror("cstatest.c: main(): tsp_open_socket(SERVER)");
		exit(-1);
	}
#endif

	/* Setup nibble memory for CSTA structs */
	InitNibbleMem(4096, 4096);
	InitAnyCSTA();

	printf("Test of the BER routines for CSTA\n");

	/* Setup Log file */
	logfile = fopen(logpath, "a");
	tm = time(NULL);
	fprintf(logfile, "\n********************************************************************************\n");
	fprintf(logfile, "\n\nRestarted on %s", ctime(&tm));
	fprintf(logfile, "\n--------------------------------------------------------------------------------\n");
	fclose(logfile);

loop:
	printf("Enter the type of primitive :\n");
	printf("\t1 - MonitorStart\n");
	printf("\t2 - MonitorStop\n");
	printf("\t3 - MakeCall\n");
	printf("\t4 - DivertCall\n");
	printf("\t5 - TranferCall\n");
	printf("\t6 - QueryDevice\n");
	printf("\t7 - AnswerCall\n");
	printf("\t8 - ACSE\n");
	printf("\n\t9 - Quit\n");

	test = getint("Your choice ->");

	switch(test) {
	case 1: /* testMonitorStart */ {
		char num[MAX_NUM_SIZE];

		getstring("Device to monitor :", num, sizeof(num));

		testMonitorStart(num);
		}
		break;

	case 2: /* testMonitorStop */ {
		unsigned int Ref;
		char RefID[MAX_NUM_SIZE];

		Ref = gethex("Monitor's crossRefID :");
		if (Ref > 255) {
			RefID[0] = (char) ((Ref >> 8) & 0x00ff);
			RefID[1] = (char) (Ref & 0x00ff);
			RefID[2] = '\0';
		} else {
			RefID[0] = (char) (Ref & 0x00ff);
			RefID[1] = '\0';
		}

		testMonitorStop(RefID);
		}
		break;
		
	case 3: /* testMakeCall */ {
 		char origin[MAX_NUM_SIZE];
		char destination[MAX_NUM_SIZE];

		getstring("Calling device :", origin, sizeof(origin));
		getstring("Called device :", destination, sizeof(destination));

		testMakeCall(origin, destination);
		}
		break;

	case 4: /* testDivertCall */ {
 		char origin[MAX_NUM_SIZE];
		char destination[MAX_NUM_SIZE];

		getstring("Device to divert from :", origin, sizeof(origin));
		getstring("Device to divert to :", destination, sizeof(destination));

		testDivertCall(origin, destination);
		}
		break;

	case 5: /* testTranferCall */ {
 		char heldDev[MAX_NUM_SIZE];
		char connDev[MAX_NUM_SIZE];

		getstring("Held device :", heldDev, sizeof(heldDev));
		getstring("Connected device :", connDev, sizeof(connDev));

		testTranferCall(heldDev, connDev);
		}
		break;

	case 6: /* testQueryDevice */ {
		char num[MAX_NUM_SIZE];
		
		getstring("Device to query :", num, sizeof(num));

		testQueryDevice(num);
		}
		break;

	case 7: /* testAnswerCall */ {
		char num[MAX_NUM_SIZE];
		unsigned int Ref;
		char RefID[5];

		getstring("Device to lift :", num, sizeof(num));

		Ref = gethex("ConnectionId:");
		RefID[0] = (char) ((Ref >> 24) & 0xff);
		RefID[1] = (char) ((Ref >> 16) & 0xff);
		RefID[2] = (char) ((Ref >> 8) & 0xff);
		RefID[3] = (char) (Ref  & 0xff);
		RefID[4] = 0;

		testAnswerCall(num, RefID);
		}
		break;
		
	case 8: /* ACSEassociation */
		ACSEassociation();
		break;

	case 9:
		end();

	default:
		printf("Bad command !\n");
		break;
	}

	goto loop;
}


void testMonitorStart(char *deviceNum)
{
	MonitorStartArgument *args;

	/* Initialize the MonitorStartArgument struct */
	args = Asn1Alloc(sizeof(MonitorStartArgument));
	/* Monitored device */
	args->monitorObject = Asn1Alloc(sizeof(MonitorObject));
	args->monitorObject->choiceId = CSTAOBJECT_DEVICE;
	args->monitorObject->a.device = Asn1Alloc(sizeof(DeviceID));
	args->monitorObject->a.device->choiceId = DEVICEID_DIALINGNUMBER;
	args->monitorObject->a.device->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->monitorObject->a.device->a.dialingNumber, deviceNum);

	/* Process APDU (fill, encode and send the APDU) */
	processAPDU((void *) args, MONITORSTARTSID);
}


void testMonitorStop(char *crossRefID)
{
	MonitorStopArgument *args;

	/* Initialize the MonitorStopArgument struct */
	args = Asn1Alloc(sizeof(MonitorStopArgument));
	/* Monitored cross reference identifier */
	args->choiceId = MONITORSTOPARGUMENT_CROSSREFIDENTIFIER;
	args->a.crossRefIdentifier = Asn1Alloc(sizeof(MonitorCrossRefID));
	setoctet(args->a.crossRefIdentifier, crossRefID);
#ifdef DEBUG_CSTATEST
	printf("CrossRefId : ");
	PrintAsnOcts(stdout, args->a.crossRefIdentifier, 0);
#endif

	/* Process APDU (fill, encode and send the APDU) */
	processAPDU((void *) args, MONITORSTOPSID);
}


void testMakeCall(char *from, char *to)
{
	MakeCallArgument *args;

	/* Initialize the MakeCallArgument struct */
	args = Asn1Alloc(sizeof(MakeCallArgument));
	/* Calling Device */
	args->callingDevice = Asn1Alloc(sizeof(DeviceID));
	args->callingDevice->choiceId = DEVICEID_DIALINGNUMBER;
	args->callingDevice->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->callingDevice->a.dialingNumber, from);
	/* Called Device */
	args->calledDirectoryNumber = Asn1Alloc(sizeof(CalledDeviceID));
	args->calledDirectoryNumber->choiceId = CALLEDDEVICEID_DEVICEIDENTIFIER;
	args->calledDirectoryNumber->a.deviceIdentifier = Asn1Alloc(sizeof(ExtendedDeviceID));
	args->calledDirectoryNumber->a.deviceIdentifier->choiceId = EXTENDEDDEVICEID_DEVICEIDENTIFIER;
	args->calledDirectoryNumber->a.deviceIdentifier->a.deviceIdentifier = Asn1Alloc(sizeof(DeviceID));
	args->calledDirectoryNumber->a.deviceIdentifier->a.deviceIdentifier->choiceId = DEVICEID_DIALINGNUMBER;
	args->calledDirectoryNumber->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->calledDirectoryNumber->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, to);

	/* Process APDU (fill, encode and send the APDU) */
	processAPDU((void *) args, MAKECALLSID);
}


void testDivertCall(char *from, char *to)
{
	DivertCallArgument *args;

	/* Initialize the DivertCallArgument struct */
	args = Asn1Alloc(sizeof(DivertCallArgument));
	/* Divert Informations */
	args->divertInfo = Asn1Alloc(sizeof(DivertInfo));
	args->divertInfo->choiceId = DIVERTINFO_DEFLECT;
	args->divertInfo->a.deflect = Asn1Alloc(sizeof(DivertInfoSeq));
	/* Originating device (FROM) */
	args->divertInfo->a.deflect->callToBeDiverted = Asn1Alloc(sizeof(ConnectionID));
	args->divertInfo->a.deflect->callToBeDiverted->device = Asn1Alloc(sizeof(ConnectionIDChoice));
	args->divertInfo->a.deflect->callToBeDiverted->device->choiceId = CONNECTIONIDCHOICE_STATICID;
	args->divertInfo->a.deflect->callToBeDiverted->device->a.staticID = Asn1Alloc(sizeof(DeviceID));
	args->divertInfo->a.deflect->callToBeDiverted->device->a.staticID->choiceId = DEVICEID_DIALINGNUMBER;
	args->divertInfo->a.deflect->callToBeDiverted->device->a.staticID->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->divertInfo->a.deflect->callToBeDiverted->device->a.staticID->a.dialingNumber, from);
	/* Destination device (TO) */
	args->divertInfo->a.deflect->newDestination = Asn1Alloc(sizeof(CalledDeviceID));
	args->divertInfo->a.deflect->newDestination->choiceId = CALLEDDEVICEID_DEVICEIDENTIFIER;
	args->divertInfo->a.deflect->newDestination->a.deviceIdentifier = Asn1Alloc(sizeof(ExtendedDeviceID));
	args->divertInfo->a.deflect->newDestination->a.deviceIdentifier->choiceId = EXTENDEDDEVICEID_DEVICEIDENTIFIER;
	args->divertInfo->a.deflect->newDestination->a.deviceIdentifier->a.deviceIdentifier = Asn1Alloc(sizeof(DeviceID));
	args->divertInfo->a.deflect->newDestination->a.deviceIdentifier->a.deviceIdentifier->choiceId = DEVICEID_DIALINGNUMBER;
	args->divertInfo->a.deflect->newDestination->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->divertInfo->a.deflect->newDestination->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, to);

	/* Process APDU (fill, encode and send the APDU) */
	processAPDU((void *) args, DIVERTCALLSID);
}


void testTranferCall(char *heldDevice, char *connDevice)
{
	TransferCallArgument *args;

	/* Initialize the TransferCallArgument struct */
	args = Asn1Alloc(sizeof(TransferCallArgument));
	args->choiceId = TRANSFERCALLARGUMENT_TRANSFERINFO;
	args->a.transferInfo = Asn1Alloc(sizeof(ConnectionDetails));
	args->a.transferInfo->choiceId = CONNECTIONDETAILS_BOTHCALLS;
	args->a.transferInfo->a.bothCalls = Asn1Alloc(sizeof(ConnectionDetailsSeq));
	/* Held device */
	args->a.transferInfo->a.bothCalls->heldCall = Asn1Alloc(sizeof(ConnectionID));
	args->a.transferInfo->a.bothCalls->heldCall->device = Asn1Alloc(sizeof(ConnectionIDChoice));
	args->a.transferInfo->a.bothCalls->heldCall->device->choiceId = CONNECTIONIDCHOICE_STATICID;
	args->a.transferInfo->a.bothCalls->heldCall->device->a.staticID = Asn1Alloc(sizeof(DeviceID));
	args->a.transferInfo->a.bothCalls->heldCall->device->a.staticID->choiceId = DEVICEID_DIALINGNUMBER;
	args->a.transferInfo->a.bothCalls->heldCall->device->a.staticID->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->a.transferInfo->a.bothCalls->heldCall->device->a.staticID->a.dialingNumber, heldDevice);
	/* Connected device */
	args->a.transferInfo->a.bothCalls->activeCall = Asn1Alloc(sizeof(ConnectionID));
	args->a.transferInfo->a.bothCalls->activeCall->device = Asn1Alloc(sizeof(ConnectionIDChoice));
	args->a.transferInfo->a.bothCalls->activeCall->device->choiceId = CONNECTIONIDCHOICE_STATICID;
	args->a.transferInfo->a.bothCalls->activeCall->device->a.staticID = Asn1Alloc(sizeof(DeviceID));
	args->a.transferInfo->a.bothCalls->activeCall->device->a.staticID->choiceId = DEVICEID_DIALINGNUMBER;
	args->a.transferInfo->a.bothCalls->activeCall->device->a.staticID->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->a.transferInfo->a.bothCalls->activeCall->device->a.staticID->a.dialingNumber, connDevice);

	/* Process APDU (fill, encode and send the APDU) */
	processAPDU((void *) args, TRANSFERCALLSID);
}


void testQueryDevice(char *deviceNum)
{
	QueryDeviceArgument *args;

	/* Initialize the QueryDeviceArgument struct */
	args = Asn1Alloc(sizeof(QueryDeviceArgument));
	/* Device */
	args->device = Asn1Alloc(sizeof(DeviceID));
	args->device->choiceId = DEVICEID_DIALINGNUMBER;
	args->device->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->device->a.dialingNumber, deviceNum);
	/* Feature */
	args->feature = DEVICEINFO;	

	/* Process APDU (fill, encode and send the APDU) */
	processAPDU((void *) args, QUERYDEVICESID);
}


void testAnswerCall(char *deviceNum, char *connid)
{
	AnswerCallArgument *args;

	/* Initialize the AnswerCallArgument struct */
	args = Asn1Alloc(sizeof(AnswerCallArgument));
	args->choiceId = ANSWERCALLARGUMENT_CALLTOBEANSWERED;
	args->a.callToBeAnswered = Asn1Alloc(sizeof(ConnectionID));
	setoctet(&args->a.callToBeAnswered->call, connid);
	args->a.callToBeAnswered->device = Asn1Alloc(sizeof(ConnectionIDChoice));
	args->a.callToBeAnswered->device->choiceId = CONNECTIONIDCHOICE_STATICID;
	args->a.callToBeAnswered->device->a.staticID = Asn1Alloc(sizeof(DeviceID));
	args->a.callToBeAnswered->device->a.staticID->choiceId = DEVICEID_DIALINGNUMBER;
	args->a.callToBeAnswered->device->a.staticID->a.dialingNumber = Asn1Alloc(sizeof(NumberDigits));
	setoctet(args->a.callToBeAnswered->device->a.staticID->a.dialingNumber, deviceNum);

	/* Process APDU (fill, encode and send the APDU) */
	processAPDU((void *) args, ANSWERCALLSID);
}


/* ACSE association management */
void ACSEassociation()
{
	ACSEUserInformationForCSTA *infos;
	SBuf encbuf;
	AsnLen len;
	ACSE_apdu pdu;
	char dummy[4] = {0x00, 0x00, 0x00, 0x00};

	/* ACSE APDU */
	pdu.choiceId = ACSE_APDU_AARQ;
	pdu.a.aarq = Asn1Alloc(sizeof(AARQ_apdu));
	/* protocol_version */
	pdu.a.aarq->protocol_version.bitLen = 2;
	pdu.a.aarq->protocol_version.bits = dummy;
	SetAsnBit(&pdu.a.aarq->protocol_version, VERSION11);
	dummy[0] = 0x00; /* clean-up dummy */
	/* application_context_name */
	pdu.a.aarq->application_context_name = csta_version2;
	/* user_information (to carry CSTA protocole information) */
	pdu.a.aarq->user_information = AsnListNew(sizeof(ACSEUserInformationForCSTA));
	infos = AsnListAppend(pdu.a.aarq->user_information);
	/* CSTA version */
	SetAsnBit(&infos->cSTAVersion, VERSIONTWO);
	/* CSTA functionalities */
	infos->cSTAFunctionsRequiredByApplication = Asn1Alloc(sizeof(CSTAFunctionality));
	infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices.bitLen = 32;
	infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices.bits = dummy;
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, ALTERNATECALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, ANSWERCALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, ASSOCIATEDATA);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, CALLCOMPLETION);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, CLEARCALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, CLEARCONNECTION);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, CONFERENCECALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, CONSULTATIONCALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, DIVERTCALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, HOLDCALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, MAKECALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, MAKEPREDICTIVECALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, PARKCALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, QUERYDEVICE);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, RECONNECTCALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, RETRIEVECALL);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, SENDDTMFTONES);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, SETFEATURE);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, SINGLESTEPCONFERENCE);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, SINGLESTEPTRANSFER);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, TRANSFERCALL);
#ifdef DEBUG_CSTATEST
	PrintAsnBits(stdout, &infos->cSTAFunctionsRequiredByApplication->switchingFunctionServices, 0);
#endif
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->eventReportServices, DELIVERED);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->computingFunctionServices, ROUTEREQUEST);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->bidirectionalServices, SYSTEMSTATUS);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->statusReportingServices, MONITORSTART);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->inputOutputServices, SENDDATASERVICE);
	SetAsnBit(&infos->cSTAFunctionsRequiredByApplication->voiceUnitServices, SUSPEND);
	/* CSTA functionalities */
	infos->cSTAFunctionsThatCanBeSupplied = Asn1Alloc(sizeof(CSTAFunctionality));
	SetAsnBit(&infos->cSTAFunctionsThatCanBeSupplied->switchingFunctionServices, MAKECALL);
	SetAsnBit(&infos->cSTAFunctionsThatCanBeSupplied->eventReportServices, DELIVERED);
	SetAsnBit(&infos->cSTAFunctionsThatCanBeSupplied->computingFunctionServices, ROUTEREQUEST);
	SetAsnBit(&infos->cSTAFunctionsThatCanBeSupplied->bidirectionalServices, SYSTEMSTATUS);
	SetAsnBit(&infos->cSTAFunctionsThatCanBeSupplied->statusReportingServices, MONITORSTART);
	SetAsnBit(&infos->cSTAFunctionsThatCanBeSupplied->inputOutputServices, SENDDATASERVICE);
	SetAsnBit(&infos->cSTAFunctionsThatCanBeSupplied->voiceUnitServices, SUSPEND);

#ifdef DEBUG_CSTATEST
	PrintAsnOid(stdout, &csta_version1, 0);
#endif
	
	/* We don't use processAPDU here because the ACSE APDU is a
	 * little bit special and uses a hack in csta2.asn1 */
	
	/* Initilialize the encbuf struct for scratch buffer */
	SBufInit(&encbuf, scratch, sizeof(scratch));
	SBufResetInWriteRvsMode(&encbuf);


	/* Encode buffer */
	len = BEncACSE_apdu(&encbuf, &pdu);
	if (SBufWriteError(&encbuf)) {
		fprintf(stderr, "\n");
		exit(-1);
	}

#ifdef DEBUG_CSTATEST
	/* SNACC pretty-print */
	printf("\n");
	PrintACSE_apdu(stdout, &pdu, 0);
	printf("\n");
#endif

	/*
	 * Free all of the decoded value since it has been encoded into the
	 * buffer. This is much more efficient than freeing each component of
	 * the value individually.
	 */
	ResetNibbleMem();

	/* Send the encoded PDU */
	if(tsp_write_socket(fdcsta, SBufDataPtr(&encbuf), SBufDataLen(&encbuf)) < 0) {
		perror("cstatest.c: ACSEassociation(): tcp_write_socket()");
		exit(-1);
	}

#ifdef DEBUG_CSTATEST
	printf("Data sent... requestID = %x\n", REQUEST_ID);
	logdump(LOG_DEBUG, SBufDataPtr(&encbuf), SBufDataLen(&encbuf));
#endif
}


/* Fill, encode and send the APDU to the CSTA server */
void processAPDU(void *args, AsnInt serviceID)
{
	ROIVapdu pdu;
	SBuf encbuf;
	AsnLen len;

	/* Initialize the pdu struct */
	pdu.invokeID = NEXT_REQUESTID;
	pdu.serviceID = serviceID;
	pdu.serviceArgs.value = args;

	/* Initilialize the encbuf struct for scratch buffer */
	SBufInit(&encbuf, scratch, sizeof(scratch));
	SBufResetInWriteRvsMode(&encbuf);

	/* Encode buffer */
	len = BEncROIVapdu(&encbuf, &pdu);
 	if (SBufWriteError(&encbuf)) {
		fprintf(stderr, "cstatest.c: processAPDU(): BEncROIVapdu(): encbuf was too small\n");
		exit(-1);
	}

#ifdef DEBUG_CSTATEST
	/* SNACC pretty-print */
	printf("\n");
	PrintROIVapdu(stdout, &pdu, 0);
	printf("\n");
#endif

	/*
	 * Free all of the decoded value since it has been encoded into the
	 * buffer. This is much more efficient than freeing each component of
	 * the value individually.
	 */
	ResetNibbleMem();

	/* Send the encoded PDU */
	if(tsp_write_socket(fdcsta, SBufDataPtr(&encbuf), SBufDataLen(&encbuf)) < 0) {
		perror("cstatest.c: testAnswerCall(): tsp_write_socket()");
		exit(-1);
	}

#ifdef DEBUG_CSTATEST
	printf("Data sent... requestID = %x\n", REQUEST_ID);
	logdump(LOG_DEBUG, SBufDataPtr(&encbuf), SBufDataLen(&encbuf));
#endif
}


/* Poll STDIN, the TCP socket connected with the CSTA server (fdcsta)
 * and the UDP socket for clients (fdserver) */
void poll_input()
{
#ifdef IS_SERVER
	struct pollfd fds[3];	
#else
	struct pollfd fds[2];	
#endif

	while (1) {
		int n;

		fds[0].fd = fdcsta;
		fds[0].events = POLLIN;
		fds[0].revents = 0;

		fds[1].fd = fileno(stdin);
		fds[1].events = POLLIN;
		fds[1].revents = 0;

#ifdef IS_SERVER
		fds[2].fd = fdserver;
		fds[2].events = POLLIN;
		fds[2].revents = 0;

		n = poll(fds, 3, INFTIM);
#else
		n = poll(fds, 2, INFTIM);
#endif
		if (n <= 0) {
			perror("cstatest.c: poll_input(): poll()");
			exit(-1);
		}

		if (fds[0].revents & POLLIN) {
			dump();
		} else if (fds[1].revents & POLLIN) {
			return;
#ifdef IS_SERVER
		} else if (fds[2].revents & POLLIN) {
			do_server();
#endif
		} else {
			perror("cstatest.c: poll_input(): unexpected non-POLLIN");
			exit(-1);
		}
	}
}


/* Initialize an octet struct from a string */
void setoctet(void *octet, char *s)
{
	int l = strlen((const char *)s);

	((AsnOcts *) octet)->octetLen = l;
	((AsnOcts *) octet)->octs = Asn1Alloc(l);
	strcpy(((AsnOcts *) octet)->octs, s);
}


/* Get a string from command line */
void getstring(char *prompt, char *buf, int size)
{
	printf("%s", prompt);
	fflush(stdout);

	poll_input();
	fgets(buf, size, stdin);

	if (buf[strlen(buf) - 1] == '\n')
		buf[strlen(buf) - 1] = '\0';
}


/* Get an integer from command line */
int getint(char *prompt)
{
	int val = 0;
	char buf[32];

	printf("%s", prompt);
	fflush(stdout);

	poll_input();
	fgets(buf, sizeof(buf), stdin);
	sscanf(buf, "%d", &val);

	return(val);
}


/* Get an hexa number from command line */
int gethex(char *prompt)
{
	int val = 0;
	char buf[32];

	printf("%s", prompt);
	fflush(stdout);

	poll_input();
	fgets(buf, sizeof(buf), stdin);
	sscanf(buf, "%x", &val);

	return(val);
}


#ifdef IS_SERVER
int do_server()
{
 	char origin[MAX_NUM_SIZE] = "800";
	char destination[MAX_NUM_SIZE];
	int n, size;

	n = tsp_read_socket(fdserver, (void *) destination, MAX_NUM_SIZE - 1);
	if (n < 0) {
		perror("csatest.c: do_server(): tsp_read_socket()");
		return(-1);
	}

	destination[n] = '\0';

	/* Logfile */
	logfile = fopen(logpath, "a");
	tm = time(NULL);
	fprintf(logfile, "\n\n%s", ctime(&tm));
	fprintf(logfile, "### MakeCall(\"%s\", \"%s\") ###", origin, destination);
	fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
	fclose(logfile);
	
	testMakeCall(origin, destination);

	return(0);
}
#endif


int dump() 
{
	SBuf sbuf;
	CSTAapdu pdu;
	AsnLen decodedLen = 0;
	jmp_buf env;
	static char buf[512];
	int error, size;

	size = tsp_read_socket(fdcsta, buf, sizeof(buf));
	if (size < 0) {
		perror("cstatest.c: dump(): tsp_read_socket()");
		return(-1);
	}

#ifdef DEBUG_CSTATEST
	printf("\n\n\n-CSTA Message------------ Len = %d\n", size);
	logdump(LOG_DEBUG, buf, size);
	printf("-------------------------\n");
#endif

	SBufInstallData(&sbuf, buf, size);

	if (!(error = setjmp(env)))
		BDecCSTAapdu(&sbuf, &pdu, &decodedLen, env);
	else {
		printf("Error in decoding pdu : %d\n", error);
		ResetNibbleMem();
		return(-1);
	}

#ifdef DEBUG_CSTATEST
	PrintCSTAapdu(stdout, &pdu, 0);
#endif

	/* Open the Log file */
	logfile = fopen(logpath, "a");

	/* Parse the APDU */
	switch (pdu.choiceId) {
	/* ServiceRequest */
	case CSTAAPDU_SVCREQUEST: {
		ROIVapdu *roivapdu = pdu.a.svcRequest;

		switch (roivapdu->serviceID) {
		/* EventReport */
		case CSTAEVENTREPORTSID: {
			CSTAEventReportArgument *event = (CSTAEventReportArgument *) (roivapdu->serviceArgs.value);

			switch (event->eventSpecificInfo->choiceId) {
			/* CallEvent */
			case EVENTSPECIFICINFO_CALLEVENT: {
				CallEvent *callevent = event->eventSpecificInfo->a.callEvent;

				/* Log time info */
				tm = time(NULL);
				fprintf(logfile, "\n\n%s", ctime(&tm));

				switch (callevent->choiceId) {
				/* CallClearedEvent */
				case CALLEVENT_CALLCLEAREDEVENT:
					fprintf(logfile, "CALLCLEAREDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* ConferencedEvent */
				case CALLEVENT_CONFERENCEDEVENT:
					fprintf(logfile, "CONFERENCEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* ConnectionClearedEvent */
				case CALLEVENT_CONNECTIONCLEAREDEVENT: {
					ConnectionClearedEvent *connclearedevent = callevent->a.connectionClearedEvent;

					fprintf(logfile, "CONNECTIONCLEAREDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);

					//PrintCorrelatorData(logfile, &connclearedevent->correlatorData, 0);
					//PrintLocalConnectionState(logfile, connclearedevent->localConnectionInfo, 0);
					//PrintEventCause(logfile, connclearedevent->cause, 0);

					fprintf(logfile, "\n-- releasingDevice -- ");
					if (connclearedevent->releasingDevice->choiceId == SUBJECTDEVICEID_DEVICEIDENTIFIER &&
					connclearedevent->releasingDevice->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
					connclearedevent->releasingDevice->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, connclearedevent->releasingDevice->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");
					}
					break;

				/* DeliveredEvent */
				case CALLEVENT_DELIVEREDEVENT: {
					DeliveredEvent *deliveredevent = callevent->a.deliveredEvent;

					fprintf(logfile, "DELIVEREDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);

					fprintf(logfile, "\n-- connectionID -- ");
					PrintAsnOcts(logfile, &deliveredevent->connection->call, 0);

					fprintf(logfile, "\n-- alertingDevice -- ");
					if (deliveredevent->alertingDevice->choiceId == SUBJECTDEVICEID_DEVICEIDENTIFIER &&
				    	deliveredevent->callingDevice->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
					deliveredevent->callingDevice->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, deliveredevent->alertingDevice->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");
				
					fprintf(logfile, "\n-- callingDevice -- ");
					if (deliveredevent->callingDevice->choiceId == CALLINGDEVICEID_DEVICEIDENTIFIER &&
				    	deliveredevent->callingDevice->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
					deliveredevent->callingDevice->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, deliveredevent->callingDevice->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");
				
					fprintf(logfile, "\n-- calledDevice -- ");
					if (deliveredevent->calledDevice->choiceId == CALLEDDEVICEID_DEVICEIDENTIFIER &&
					deliveredevent->callingDevice->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
				    	deliveredevent->callingDevice->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, deliveredevent->calledDevice->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");
				
					fprintf(logfile, "\n-- lastRedirectionDevice -- ");
					if (deliveredevent->lastRedirectionDevice->choiceId == REDIRECTIONDEVICEID_NUMBERDIALED &&
					deliveredevent->lastRedirectionDevice->a.numberdialed->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
					deliveredevent->lastRedirectionDevice->a.numberdialed->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, deliveredevent->lastRedirectionDevice->a.numberdialed->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");
					}
					break;

				/* DivertedEvent */
				case CALLEVENT_DIVERTEDEVENT: {
					DivertedEvent *divertedevent = callevent->a.divertedEvent;

					fprintf(logfile, "DIVERTEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
				
					fprintf(logfile, "\n-- divertingDevice -- ");
					if (divertedevent->divertingDevice->choiceId == SUBJECTDEVICEID_DEVICEIDENTIFIER &&
					divertedevent->divertingDevice->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
					divertedevent->divertingDevice->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, divertedevent->divertingDevice->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");
				
					fprintf(logfile, "\n-- newDestination -- ");
					if (divertedevent->newDestination->choiceId == SUBJECTDEVICEID_DEVICEIDENTIFIER &&
					divertedevent->newDestination->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
					divertedevent->newDestination->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, divertedevent->newDestination->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");
					}
					break;

				/* EstablishedEvent */
				case CALLEVENT_ESTABLISHEDEVENT:
					fprintf(logfile, "ESTABLISHEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* FailedEvent */
				case CALLEVENT_FAILEDEVENT:
					fprintf(logfile, "FAILEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* HeldEvent */
				case CALLEVENT_HELDEVENT:
					fprintf(logfile, "HELDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* NetworkReachedEvent */
				case CALLEVENT_NETWORKREACHEDEVENT:
					fprintf(logfile, "NETWORKREACHEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* OriginatedEvent */
				case CALLEVENT_ORIGINATEDEVENT:
					fprintf(logfile, "ORIGINATEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* QueuedEvent */
				case CALLEVENT_QUEUEDEVENT:
					fprintf(logfile, "QUEUEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* RetrievedEvent */
				case CALLEVENT_RETRIEVEDEVENT:
					fprintf(logfile, "RETRIEVEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* ServiceInitiatedEvent */
				case CALLEVENT_SERVICEINITIATEDEVENT:
					fprintf(logfile, "SERVICEINITIATEDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				/* TransferedEvent */
				case CALLEVENT_TRANSFEREDEVENT:
					fprintf(logfile, "TRANSFEREDEVENT ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;

				default:
					fprintf(logfile, "Not a CALLEVENT_[...] ");
					PrintMonitorCrossRefID(logfile, &event->crossRefIdentifier, 0);
					break;
				}

				fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
			}
			break;

			/* FeatureEvent */
			case EVENTSPECIFICINFO_FEATUREEVENT: {
				FeatureEvent  *featureevent = event->eventSpecificInfo->a.featureEvent;
			   
				switch (featureevent->choiceId) {
				/* ForwardingEvent */
				case FEATUREEVENT_FORWARDINGEVENT: {
					ForwardingEvent *forwardingevent = featureevent->a.forwardingEvent;

					tm = time(NULL);
					if (forwardingevent->forwardingInformation->forwardingType == 0) {
						fprintf(logfile, "\n\nNew forward on %s", ctime(&tm));
					} else if (forwardingevent->forwardingInformation->forwardingType == 1) {
						fprintf(logfile, "\n\nEnd of forward on %s", ctime(&tm));
					}

					fprintf(logfile, "\nFORWARDINGEVENT");

					fprintf(logfile, "\n-- forwardingType -- ");
					PrintForwardingType(logfile, (AsnInt *) &forwardingevent->forwardingInformation->forwardingType, 0);

					fprintf(logfile, "\n-- forwardedDeviceNumber -- ");
					PrintNumberDigits(logfile, &forwardingevent->forwardingInformation->forwardDN, 0);

					fprintf(logfile, "\n-- forwardedDevice -- ");
					if (forwardingevent->device->choiceId == SUBJECTDEVICEID_DEVICEIDENTIFIER &&
				    	forwardingevent->device->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
					forwardingevent->device->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
						PrintNumberDigits(logfile, forwardingevent->device->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
					else
						fprintf(logfile, "''H -- \"\" --");

					if (forwardingevent->forwardedTo != NULL) {
						fprintf(logfile, "\n-- forwardedToDevice -- ");

						if (forwardingevent->forwardedTo->choiceId == SUBJECTDEVICEID_DEVICEIDENTIFIER &&
				    		forwardingevent->forwardedTo->a.deviceIdentifier->choiceId == EXTENDEDDEVICEID_DEVICEIDENTIFIER &&
						forwardingevent->forwardedTo->a.deviceIdentifier->a.deviceIdentifier->choiceId == DEVICEID_DIALINGNUMBER)
							PrintNumberDigits(logfile, forwardingevent->forwardedTo->a.deviceIdentifier->a.deviceIdentifier->a.dialingNumber, 0);
						else
							fprintf(logfile, "''H -- \"\" --");
					}

					fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
				}
				break;

				/* !ForwardingEvent */
				default:
					fprintf(logfile, "\n\nNot a FEATUREEVENT_FORWARDINGEVENT");
					fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
					break;
				}
			}
			break;

			/* !(CallEvent or FeatureEvent) */
			default:
				fprintf(logfile, "\n\nNot a EVENTSPECIFICINFO_[CALLEVENT,FEATUREEVENT]");
				fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
				break;
			}
		}
		break;

		case SYSTEMSTATUSSID:
		{
			SystemStatusArgument *event = (SystemStatusArgument *) (roivapdu->serviceArgs.value);

			switch  (event->choiceId)
			{
			case SYSTEMSTATUSARGUMENT_SYSTEMSTATUS:
				fprintf(logfile, "\n\nSystem status = %d", event->a.systemStatus);
				fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
				break;
			case SYSTEMSTATUSARGUMENT_SYSTEMSTATUSARGUMENTSEQ:
				break;
			default:
				fprintf(logfile, "\n\nNot a SYSTEMSTATUSSID\n");
				fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
			break;
			}
			fprintf(logfile, "\n\nSYSTEMSTATUSSID\n");
		}
			break;

		/* !EventReport */
		default:
			fprintf(logfile, "\n\nNot a CSTAEVENTREPORTSID\n");
			fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
			break;
		}
	}
	break;

	/* !ServiceRequest */
	default:
		fprintf(logfile, "\n\nNot a CSTAAPDU_SVCREQUEST\n");
		fprintf(logfile, "\n\n--------------------------------------------------------------------------------\n");
		break;
	}

	/* Close the Log file to avoid loosing datas */
	fclose(logfile);

#ifdef DEBUG_CSTATEST
	fprintf(stdout, "\n-----------------------------");
	fprintf(stdout, "\n-----------------------------");
	fprintf(stdout, "\n-----------------------------\n");
#endif

	ResetNibbleMem();

	return(0);
}


/* Handler for exit signals */
void end()
{
	printf("\n\nEnd of cstatest\n");

	/* Close Log file */
	logfile = fopen(logpath, "a");
	tm = time(NULL);
	fprintf(logfile, "\n\nStopped on %s\n", ctime(&tm));
	fclose(logfile);

	/* Close socket descriptors */
	close(fdcsta);
#ifdef IS_SERVER
	tsp_close_socket(fdserver);
#endif

	/* Free the NibbleMemory slice */
	ShutdownNibbleMem();

	exit(0);
}
