#!/usr/bin/perl -w

# Copyright (C) 2000 Herv Eychenne, Julien Gaulmin et Alcve
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Telephone;

use strict;
use DBI;

require Exporter;
use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(init end get_contact_from_phone get_organisation_from_phone);

my ($dbh, $sth, $acces_ref);
my $dbname = "alcove_test";

sub init {
	#my $USER = getlogin() || getpwuid($<) || $ENV{USER};
	my $USER = "xortex";
	$dbh = DBI->connect("DBI:Pg:dbname=$dbname;host=db-alcove",
		"$USER", '') || die "$DBI::errstr";
}

sub end {
	if (defined $sth) {
		$sth->finish;
	}

	$dbh->disconnect;
}

sub get_name_from_organisation {
	my ($id) = @_;
	my $sth;

	$sth = $dbh->prepare("SELECT nom FROM organisations WHERE id = $id")
		|| die "$DBI::errstr";

	$sth->execute || die "$DBI::errstr";

	return ($sth->fetchrow_array)[0];
}

sub get_contact_from_phone {
	my ($phone) = @_;
	my $count = 0;

	$sth = $dbh->prepare("SELECT * FROM contacts WHERE tlphone1 ~ '$phone'
		OR tlphone2 ~ '$phone'") || die "$DBI::errstr";

	$sth->execute || die "$DBI::errstr";

	while (my $hash_ref = $sth->fetchrow_hashref) {
		$count++;

		printf "%-14s  %-14s  %-25s  %-14s   %-14s\n",
		$hash_ref->{'prnom'} ? $hash_ref->{'prnom'} : "",
		$hash_ref->{'nom'},
		get_name_from_organisation($hash_ref->{'organisation'}),
		$hash_ref->{'tlphone1'},
		$hash_ref->{'tlphone2'} ? $hash_ref->{'tlphone2'} : "";
	}

	return $count;
}

sub get_contact_name_from_phone {
	my ($phone) = @_;
	my $count = 0;

	# Regex pour s'adapter au format non-uniforme
	$phone =~ s/(\d\d)/$1\.\*/g;

	$sth = $dbh->prepare("SELECT * FROM contacts WHERE tlphone1 ~ '$phone'
		OR tlphone2 ~ '$phone'") || die "$DBI::errstr";

	$sth->execute || die "$DBI::errstr";

	my $hash_ref = $sth->fetchrow_hashref;

	if (defined $hash_ref) {
		my $prenom = $hash_ref->{'prnom'} ? $hash_ref->{'prnom'} : "";
		return "$hash_ref->{'nom'} $prenom";
	} else {
		return "???";
	}

	return $count;
}

sub get_organisation_from_phone {
	my ($phone) = @_;
	my $count = 0;

	$sth = $dbh->prepare("SELECT * FROM organisations WHERE tlphone ~ '$phone'
		OR fax ~ '$phone'") || die "$DBI::errstr";

	$sth->execute || die "$DBI::errstr";

	while (my $hash_ref = $sth->fetchrow_hashref) {
		$count++;

		printf "%-25s   %-14s   %-14s\n",
		$hash_ref->{'nom'},
		$hash_ref->{'tlphone'},
		$hash_ref->{'fax'} ? $hash_ref->{'fax'} : "";
	}

	return $count;
}

sub get_organisation_id_from_phone {
	my ($phone) = @_;

	# Regex pour s'adapter au format non-uniforme
	$phone =~ s/(\d\d)/$1\.\*/g;

	$sth = $dbh->prepare("SELECT * FROM organisations WHERE tlphone ~ '$phone'
		OR fax ~ '$phone'") || die "$DBI::errstr";

	$sth->execute || die "$DBI::errstr";

	my $hash_ref = $sth->fetchrow_hashref;

	if (defined $hash_ref) {
		return $hash_ref->{'id'};
	} else {
		$sth = $dbh->prepare("SELECT * FROM contacts WHERE tlphone1 ~ '$phone'
			OR tlphone2 ~ '$phone'") || die "$DBI::errstr";

		$sth->execute || die "$DBI::errstr";

		$hash_ref = $sth->fetchrow_hashref;

		if (defined $hash_ref) {
			return $hash_ref->{'organisation'};
		} else {
			return 0;
		}
	}
}

1;

# vim: ts=2 sw=2
