#!/usr/bin/perl -w

# Copyright (C) 2000 Alcve and Julien Gaulmin <julien.gaulmin@fr.alcove.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Version 2
#
# Arguments :
#	TEL = numro de tlphone au format interne (3 derniers chiffres),
#	DATE = date en anglais (ex : today, yesterday 16:00...).

use strict;
use CGI qw(:standard);
use Date::Manip;
use Telephone;

my $num = param('TEL');
my $date_limit = param('DATE');

my $numtarget;
my $org;
my $date_month;
my $date_day;
my $date_hour;
my $date_min;
my $EPOCH_start;
my $EPOCH_stop;
my $EPOCH_limit;
my $last;

my $log_file = "/import/cti/cti-data/csta.out";
my $header_file = "/home/julien23/src/alcove/cti/src/cstatest/web/header_html";
my $footer_file = "/home/julien23/src/alcove/cti/src/cstatest/web/footer_html";

# Par dfaut
$num = 775 unless defined $num;
$date_limit = "today" unless defined $date_limit;

# Header HTML
print header();
display($header_file);

# Calcul de la date
error("Mauvaise date !") unless ParseDate($date_limit);
$EPOCH_limit = UnixDate(ParseDate($date_limit), "%a %b %e");

# Connexion  la base de donnes Alcve
Telephone::init();

# Parsing du fichier de log
open(LOG, "$log_file") or error("Unable to open $log_file");

# Positionnement  la date de dbut dans le log
if (defined $date_limit) {
	my $found = 0;

	while (<LOG>) {
		if (/^$EPOCH_limit/) {
			$found = 1;
			$last = $_;
			last;
		}
	}
}

# Machine  tats
my $state = 0;

while (<LOG>) {

	# SONNNERIE (appel entrant/sortant)
	if (/^DELIVEREDEVENT '${num}f'H/) {
		debug("DELIVEREDEVENT\n$last");

		if ($state == 0) {
			($date_month, $date_day, $date_hour, $date_min, $EPOCH_start) =
			UnixDate(ParseDate($last), ("%m", "%d", "%H", "%M", "%s"));

			# On saute la ligne ConnectionID
			$_ = <LOG>;

			# On saute la ligne AlertingDevice
			$_ = <LOG>;

			# Dtermine le type d'appel (entrant/sortant)
			if (<LOG> =~ /^-- callingDevice -- '\d+'H *-- "(\d+)"/) {
				# Appel non masqu

				$numtarget = $1;

				if ($numtarget == $num) {
					# Appel SORTANT

					# Evaluation du numro de l'appel
					if (<LOG> =~ /^-- calledDevice -- '\d+'H *-- "(\d+)"/) {
						$numtarget = $1;
					} else {
						# FIX ME : ce cas existe-t-il ?
						$numtarget = "inconnu (FIX ME)";
					}

					print	"  <TR bgcolor=#AFAFFF>\n" .
						"    <TD align=center>S</TD>\n";
				} else {
					# Appel ENTRANT

					print	"  <TR bgcolor=#AFFFAF>\n" .
						"    <TD align=center>E</TD>\n";
				}

				# Mise en forme du numro de tlphone
				$numtarget =~ s/^0+([1-9])/0$1/;

				# Recherche du correspondant dans la base Alcve
				if (length($numtarget) > 3 and
						$org = Telephone::get_organisation_id_from_phone($numtarget)) {

					my $url = 'http://db-alcove.alcove-fr/organisation.cgi' .
						'?selection=S%E9lectionner&SELECTION_WHERE_0=' . $org;
					my $org_name = Telephone::get_name_from_organisation($org);
					my $contact_name = Telephone::get_contact_name_from_phone($numtarget);

					$numtarget = "$numtarget " .
						"(<A href=\"$url\">$org_name</A>, ".
						"<A href=\"$url\">$contact_name</A>)";
				}
			} else {
				# Appel entrant anonyme
				$numtarget = "inconnu";

				print	"  <TR bgcolor=#AFFFAF>\n    <TD align=center>E</TD>\n";
			}

			print	"    <TD>$date_day/$date_month</TD>\n" .
				"    <TD>$date_hour:$date_min</TD>\n    <TD>$numtarget</TD>\n";

			$state++;
		} else {
			# Etat non trait
			debug("DELIVEREDEVENT\n${last}Etat $state non trait");
		}

		next;
	}

	# REDIRECTION
  if (/^DIVERTEDEVENT '${num}f'H/) {
		debug("DIVERTEDEVENT\n$last");

		# On saute la ligne divertingDevice
		$_ = <LOG>;

		<LOG> =~ /^-- newDestination -- '\d*'H  -- "(\d*)"/;

		if ($state == 0) {
			($date_month, $date_day, $date_hour, $date_min, $EPOCH_start) =
				UnixDate(ParseDate($last), ("%m", "%d", "%H", "%M", "%s"));

			# Redirection immdiate
			print	"  <TR bgcolor=#AFFFAF>\n" .
				"    <TD align=center>E</TD>\n" .
				"    <TD>$date_day/$date_month</TD>\n" .
				"    <TD>$date_hour:$date_min</TD>\n" .
				"    <TD colspan=3 align=center><B>Redirig automatiquement vers $1</B></TD>\n" .
				"  </TR>\n";

			$state = 0;
		} elsif ($state == 1) {
			# Appel redirig
			print	"    <TD>&nbsp;</TD>\n" .
				"    <TD><B>Redirig automatiquement vers $1</B></TD>\n" .
				"  </TR>\n";

			$state = 0;
		} else {
			# Etat non trait
			debug("DIVERTEDEVENT\n${last}Etat $state non trait");
		}

		next;
  }

	# TRANSFERT
	if (/^TRANSFEREDEVENT '${num}f'H/) {
		debug("TRANSFEREDEVENT\n$last");

		if ($state == 2) {
			# Appel transfr

			($EPOCH_stop) = UnixDate(ParseDate($last), ("%s"));

			printf "    <TD><B>Transfr aprs %d\' %d\'\'</B></TD>\n  </TR>\n",
				($EPOCH_stop-$EPOCH_start)/60, ($EPOCH_stop-$EPOCH_start)%60;

			$state = 0;
		} else {
			# Etat non trait
			debug("TRANSFEREDEVENT\n${last}Etat $state non trait");
		}

		next;
	}

	# RACROCHAGE
	if (/^CONNECTIONCLEAREDEVENT '${num}f'H/) {
		debug("CONNECTIONCLEAREDEVENT\n$last");

		if ($state == 1) {
			# Appel non dcroch

			($EPOCH_stop) = UnixDate(ParseDate($last), ("%s"));

			printf	"    <TD>%d\' %d\'\'</TD>\n",
				($EPOCH_stop-$EPOCH_start)/60, ($EPOCH_stop-$EPOCH_start)%60;

			if (length($numtarget) > 3) {
				print	"    <TD bgcolor=#FFAFAF><B>Non dcroch</B></TD>\n  </TR>\n";
			} else {
				print	"    <TD><B>Non dcroch</B></TD>\n  </TR>\n";
			}

			$state = 0;
		} elsif ($state == 2) {
			# Fin d'appel normale

			($EPOCH_stop) = UnixDate(ParseDate($last), ("%s"));

			printf	"    <TD>%d\' %d\'\'</TD>\n  </TR>\n",
				($EPOCH_stop-$EPOCH_start)/60, ($EPOCH_stop-$EPOCH_start)%60;

			$state = 0;
		} else {
			# Etat non trait
			debug("CONNECTIONCLEAREDEVENT\n${last}Etat $state non trait");
		}

		next;
	}

	# DCROCHAGE
	if (/^ESTABLISHEDEVENT '${num}f'H/) {
		debug("ESTABLISHEDEVENT\n$last");

		if ($state == 1) {
			# Appel tabli

			($EPOCH_stop) = UnixDate(ParseDate($last), ("%s"));

			printf	"    <TD>%d\' %d\'\'</TD>\n",
				($EPOCH_stop-$EPOCH_start)/60, ($EPOCH_stop-$EPOCH_start)%60;

			# Heure de dbut de connexion
			$EPOCH_start = $EPOCH_stop;

			$state++;
		} else {
			# Etat non trait
			debug("ESTABLISHEDEVENT\n${last}Etat $state non trait");
		}

		next;
	}

	# On conserve toujours la ligne prcdente pour la date
	$last = $_;
}
close(LOG);

# Footer HTML
display($footer_file);

# Deconnexion de la base de donnes Alcve
Telephone::end();

#------------------------------------------------------------------------------
# My functions
#------------------------------------------------------------------------------
sub display {
	my $filename = $_[0];

	unless (open(DISPLAYSTREAM, "$filename")) {
		print "Unable to display $filename\n";
		return;
	}
	while (<DISPLAYSTREAM>) {
		s/\$num/$num/g;
		print $_;
	}
	close(DISPLAYSTREAM);
}

sub error {
	my $string = $_[0];

	print	"  <TR>\n    <TD colspan=6 bgcolor=#FFAFAF>" .
		"<B>ERROR : parser_tel.cgi : $string</B></TD>\n  </TR>\n";
	display($footer_file);
	die;
}

sub debug {
	my $string = $_[0];

	print	"<!--\n*****\n$string\n*****\n-->\n";
}

# vim: ts=2 sw=2
